import tensorflow as tf
import numpy as np
import matplotlib.pyplot as plt

# Load an image
image = tf.keras.preprocessing.image.load_img('images/dog.jpg', target_size=(224, 224))
image = tf.keras.preprocessing.image.img_to_array(image)

# Preprocess the image
image = tf.keras.applications.mobilenet_v2.preprocess_input(image)

# Load the MobileNetV2 model
model = tf.keras.applications.MobileNetV2()

# Define the layer to visualize
layer_name = 'block_1_expand_relu'

# Define a function to get the activation maps for a given layer and image
def get_activation_maps(model, layer_name, image):
  # Create a sub-model that outputs the activations for the given layer
  outputs = [model.get_layer(layer_name).output]
  sub_model = tf.keras.models.Model(inputs=model.inputs, outputs=outputs)

  # Get the activations for the image
  activations = sub_model.predict(np.array([image]))

  # Return the activations
  return activations[0]

# Get the activation maps for the defined layer
activations = get_activation_maps(model, layer_name, image)

# Plot the activation maps
fig, axes = plt.subplots(4, 4, figsize=(8, 8))
for i, ax in enumerate(axes.flat):
    ax.imshow(activations[..., i], cmap='jet')
    ax.set_axis_off()
plt.savefig('interpretation_methods/activation_maps/output/dog/activation_maps_' + layer_name + '.png')
plt.show()