import tensorflow as tf
import matplotlib.pyplot as plt

# Load the MobileNetV2 model
model = tf.keras.applications.MobileNetV2()

# Define the layer to visualize
layer_name = 'Conv1'
layer = model.get_layer(name=layer_name)

# Define the size of the generated image
img_size = 224

# Define the number of iterations and the step size for optimization
iterations = 100
step_size = 1

# Define the loss function to maximize the activation of the chosen layer
def activation_loss(layer, img):
    activation = layer(img)
    return tf.reduce_mean(activation)

# Define the function to generate the image by maximizing the activation of the chosen layer
def generate_image(layer, img_size, iterations, step_size):
    # Initialize the image with random noise
    img = tf.Variable(tf.random.uniform((1, img_size, img_size, 3), 0.0, 1.0))

    # Display random noise image
    plt.imshow(img.numpy()[0], cmap='jet')
    plt.savefig('interpretation_methods/activation_maximization/output/activation_maximization_' + layer_name + '_generated' +'.png')
    plt.show()

    # Define the optimizer
    optimizer = tf.keras.optimizers.Adam(learning_rate=step_size)

    # Run the optimization for the specified number of iterations
    for i in range(iterations):
        with tf.GradientTape() as tape:
            # Compute the loss
            loss = activation_loss(layer, img)

        # Compute the gradients of the loss with respect to the image
        gradients = tape.gradient(loss, img)
        gradients /= tf.sqrt(tf.reduce_mean(tf.square(gradients))) + 1e-5

        # Update the image
        optimizer.apply_gradients([(gradients, img)])

        # Clip the image to keep the pixel values within the [0, 1] range
        img.assign(tf.clip_by_value(img, 0.0, 1.0))

    # Return the generated image
    return img.numpy()[0]

# Generate the image by maximizing the activation of the chosen layer
img = generate_image(layer, img_size, iterations, step_size)

# Display the generated image
plt.imshow(img, cmap='jet')
plt.savefig('interpretation_methods/activation_maximization/output/activation_maximization_' + layer_name + '.png')
plt.show()
