import tensorflow as tf
import matplotlib.pyplot as plt

# Load the neural network model and the image size
model = tf.keras.applications.VGG16()
img_size = 224

# Define the target classification class
target_class = 156 

# Define the number of iterations and the step size for optimization
iterations = 1000
step_size = 0.1

# Define the loss function to maximize the output of the target classification class
def activation_loss(model, img, target_class):
    output = model(img)
    target_output = output[:, target_class]
    return tf.reduce_mean(target_output)

# Define the function to generate the image by maximizing the activation of the entire network based on the target classification class
def generate_image(model, img_size, target_class, iterations, step_size):
    # Initialize the image with random noise
    img = tf.Variable(tf.random.uniform((1, img_size, img_size, 3), 0.0, 1.0))

    plt.imshow(img.numpy()[0])
    plt.savefig('interpretation_methods/activation_maximization/output/VGG16/activation_maximization_whole_network_start_' + str(target_class) + '.png')
    plt.show()

    # Define the optimizer
    optimizer = tf.keras.optimizers.Adam(learning_rate=step_size)

    # Run the optimization for the specified number of iterations
    for i in range(iterations):
        with tf.GradientTape() as tape:
            # Compute the loss
            loss = activation_loss(model, img, target_class)
            print(str(loss))
            print(str(i))

        # Compute the gradients of the loss with respect to the image
        gradients = tape.gradient(loss, img)
        gradients /= tf.sqrt(tf.reduce_mean(tf.square(gradients))) + 1e-5

        # Update the image
        optimizer.apply_gradients([(gradients, img)])

        # Clip the image to keep the pixel values within the [0, 1] range
        img.assign(tf.clip_by_value(img, 0.0, 1.0))

    # Return the generated image
    return img.numpy()[0]

# Generate the image by maximizing the activation of the entire network based on the target classification class
img = generate_image(model, img_size, target_class, iterations, step_size)

# Display the generated image
plt.imshow(img, cmap='jet')
plt.savefig('interpretation_methods/activation_maximization/output/VGG16/activation_maximization_whole_network_end_' + str(target_class) + '.png')
plt.show()