import tensorflow as tf
import numpy as np
import matplotlib.pyplot as plt

# Load an image
image = tf.keras.preprocessing.image.load_img('images/dog.jpg', target_size=(224, 224))
image = tf.keras.preprocessing.image.img_to_array(image)

# Preprocess the image
image = tf.keras.applications.mobilenet_v2.preprocess_input(image)

# Load the neural network model
model = tf.keras.applications.MobileNetV2()

# Define the target classification class
target_class = 156 

# Define the number of iterations and the step size for optimization
iterations = 10000
step_size = 0.1

# Define the loss function to maximize the output of the target classification class
def activation_loss(model, image, target_class):
    output = model(image)
    target_output = output[:, target_class]
    return tf.reduce_mean(target_output)

# Define the function to generate the image by maximizing the activation of the entire network based on the target classification class
def generate_image(model, image, target_class, iterations, step_size):
    # Add a batch dimension to the image tensor
    image = np.expand_dims(image, axis=0)
    # Initialize the image
    image = tf.Variable(image)

    plt.imshow(image.numpy()[0])
    plt.savefig('interpretation_methods/activation_maximization/output/MobileNetV2/activation_maximization_whole_network_image_start_' + str(target_class) + '.png')
    plt.show()

    # Define the optimizer
    optimizer = tf.keras.optimizers.Adam(learning_rate=step_size)

    # Run the optimization for the specified number of iterations
    for i in range(iterations):
        with tf.GradientTape() as tape:
            # Compute the loss
            loss = activation_loss(model, image, target_class)
            print(str(loss))
            print(str(i))

        # Compute the gradients of the loss with respect to the image
        gradients = tape.gradient(loss, image)
        gradients /= tf.sqrt(tf.reduce_mean(tf.square(gradients))) + 1e-5

        # Update the image
        optimizer.apply_gradients([(gradients, image)])

        # Clip the image to keep the pixel values within the [0, 1] range
        image.assign(tf.clip_by_value(image, 0.0, 1.0))

    # Return the generated image
    return image.numpy()[0]

# Generate the image by maximizing the activation of the entire network based on the target classification class
img = generate_image(model, image, target_class, iterations, step_size)

# Display the generated image
plt.imshow(img, cmap='jet')
plt.savefig('interpretation_methods/activation_maximization/output/MobileNetV2/activation_maximization_whole_network_image_end_' + str(target_class) + '.png')
plt.show()