import tensorflow as tf
import numpy as np
import matplotlib.pyplot as plt

# Load an image
image = tf.keras.preprocessing.image.load_img('images/dog.jpg', target_size=(224, 224))
image = tf.keras.preprocessing.image.img_to_array(image)

# Preprocess the image
image = tf.keras.applications.mobilenet_v2.preprocess_input(image)

# Load the MobileNetV2 model
model = tf.keras.applications.MobileNetV2()

# Define occlusion sensitivity function
def occlusion_sensitivity(model, image, target_class, patch_size):
    # Get the predicted probabilities for the target class
    probs = model.predict(np.array([image]))
    target_prob = probs[0][target_class]
    
    # Initialize an empty heatmap
    heatmap = np.zeros(image.shape[:2])
    
    # Loop over patches of the image
    for row in range(0, image.shape[0], patch_size):
        for col in range(0, image.shape[1], patch_size):
            print('Row is ' + str(row))
            # Copy the original image
            occluded_image = np.copy(image)
            
            # Occlude the current patch
            occluded_image[row:row+patch_size, col:col+patch_size, :] = 0
            
            # Get the predicted probabilities for the occluded image
            probs = model.predict(np.array([occluded_image]))
            occluded_prob = probs[0][target_class]
            
            # Compute the difference in probabilities
            prob_diff = target_prob - occluded_prob
            
            # Update the heatmap with the probability difference
            heatmap[row:row+patch_size, col:col+patch_size] = prob_diff
    
    # Normalize the heatmap
    heatmap = (heatmap - np.min(heatmap)) / (np.max(heatmap) - np.min(heatmap) + 1e-10)
    
    return heatmap

# Get the occlusion sensitivity heatmap
target_class = 156 # class index
patch_size = 1 # size of a patch
heatmap = occlusion_sensitivity(model, image, target_class, patch_size)

# Plot the original image and the heatmap
fig, ax = plt.subplots(1, 3, figsize=(15, 5))
ax[0].imshow(image)
ax[0].axis('off')
ax[0].set_title('Original Image')
ax[1].imshow(heatmap, cmap='jet', alpha=0.5)
ax[1].imshow(image, alpha=0.5)
ax[1].axis('off')
ax[1].set_title('Occlusion Sensitivity Heatmap')
ax[2].imshow(heatmap, cmap='jet')
ax[2].axis('off')
ax[2].set_title('Heatmap')
plt.savefig('interpretation_methods/occlusion_sensitivity/output/occlusion_sensitivity_'+ str(patch_size)+'.png')
plt.show()
