import tensorflow as tf
import numpy as np
import matplotlib.pyplot as plt

# Load an image
image = tf.keras.preprocessing.image.load_img('images/spaniel/spaniel.jpg', target_size=(224, 224))
image = tf.keras.preprocessing.image.img_to_array(image)

image1 = tf.keras.preprocessing.image.load_img('images/spaniel/spaniel1.jpg', target_size=(224, 224))
image1 = tf.keras.preprocessing.image.img_to_array(image1)

image2 = tf.keras.preprocessing.image.load_img('images/spaniel/spaniel2.jpg', target_size=(224, 224))
image2 = tf.keras.preprocessing.image.img_to_array(image2)

image3 = tf.keras.preprocessing.image.load_img('images/spaniel/spaniel3.jpg', target_size=(224, 224))
image3 = tf.keras.preprocessing.image.img_to_array(image3)

# Preprocess the image
image = tf.keras.applications.mobilenet_v2.preprocess_input(image)
image1 = tf.keras.applications.mobilenet_v2.preprocess_input(image1)
image2 = tf.keras.applications.mobilenet_v2.preprocess_input(image2)
image3 = tf.keras.applications.mobilenet_v2.preprocess_input(image3)

# Load the MobileNetV2 model
model = tf.keras.applications.MobileNetV2()

# Define a function to get the saliency maps for a given batch of images and class index
def get_saliency_maps(model, images, class_index):
  # Get the gradient of the class output with respect to the input image
  input_images = tf.cast(images, tf.float32)
  with tf.GradientTape() as tape:
    tape.watch(input_images)
    output = model(input_images)
    class_output = output[:, class_index]
  gradient = tape.gradient(class_output, input_images)

  # Compute the saliency maps
  saliency_maps = tf.math.reduce_max(tf.math.abs(gradient), axis=-1)

  # Normalize the saliency maps
  saliency_maps /= tf.math.reduce_max(saliency_maps, axis=(1, 2), keepdims=True)

  # Convert the saliency maps to numpy arrays
  saliency_maps = saliency_maps.numpy()

  # Return the saliency maps
  return saliency_maps

# Create a batch of images
images = np.array([image, image1, image2, image3])

# Get the saliency maps for defined class
class_index = 156
saliency_maps = get_saliency_maps(model, images, class_index)

#Plot preprocessed original images
fig, axes = plt.subplots(2, 2, figsize=(8, 8))
for i, ax in enumerate(axes.flat):
    ax.imshow(images[i])
    ax.set_axis_off()
plt.savefig('interpretation_methods/saliency_maps/output/original_images_spaniel_' + str(class_index) + '.png')
plt.show()

# Plot the saliency maps
fig, axes = plt.subplots(2, 2, figsize=(8, 8))
for i, ax in enumerate(axes.flat):
    ax.imshow(saliency_maps[i], cmap='jet')
    ax.set_axis_off()
plt.savefig('interpretation_methods/saliency_maps/output/saliency_maps_spaniel_' + str(class_index) + '.png')
plt.show()
